==> wyvern-2.2.7.tar.bz2: wyvern-2.2.7/modules/mod_env.c <==
/*****************************************************************************
 *
 * FILE:	mod_env.c
 * DESCRIPTION:	Wyvern: makeing environment variables module
 * DATE:	Tue, Aug 20 2002
 * UPDATED:	Fri, Mar  3 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2006 졿Kouichi ABE (WALL), All rights reserved.
 * REFERENCE:	draft-coar-cgi-v11-03.txt
 * LICENSE:
 *
 *  Copyright (c) 2002-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_env.c,v 1.24 2006/03/03 12:21:23 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <pwd.h>
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structure definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_env"

#define	length(x)	(sizeof((x)) - 1)

/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	wm_env_init(void);
static void	wm_env_finish(void);
static int	wm_set_script_environment(Message *);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static char **	env;
static int	envno;

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_env_init
 * Description:		initialize server environment for CGI/SSI
 * Arguments:		none
 * Return Value:	none
 * Notes:
 *
 * SERVER_SOFTWARE:	the name and version of the server software
 * SERVER_NAME:		the name of the server
 * GATEWAY_INTERFACE:	the revision of the CGI specification
 *
 * SERVER_PROTOCOL:	the protocol version of the HTTP
 * SERVER_PORT:		the port which the request is received on the server
 *
 * REQUEST_METHOD:	the method of the HTTP
 * SCRIPT_NAME:		the identity of the CGI script
 * PATH_INFO:		the path to be interpreted by the CGI script
 * PATH_TRANSLATED:	the location in the server's document
 * QUERY_STRING:	a URL-encoded string
 * REMOTE_HOST:		the FQDN of the client
 * REMOTE_ADDR:		the IP address of the client
 * REMOTE_PORT:		the remote port of the client
 * AUTH_TYPE:		the requests "Authorization"
 * REMOTE_USER:		the remote user-ID
 * REMOTE_IDENT:	the identity information reported about the connection
 *			by a RFC 1413
 * CONTENT_TYPE:	the Internet Media Type
 * CONTENT_LENGTH:	the size of the message-body entity
 *
 *
 * HTTP_ACCEPT:
 * HTTP_ACCEPT_ENCODING:
 * HTTP_ACCEPT_LANGUAGE:
 * HTTP_HOST:
 * HTTP_USER_AGENT:
 * HTTP_REFERER:
 * HTTP_COOKIE:
 * HTTP_COOKIE2:
 *
 * DOCUMENT_ROOT:	NCSA and APACHE added -- not in CGI spec
 * SERVER_ROOT:		NCSA added
 * SERVER_ADMIN:	APACHE added
 */
static void
wm_env_init(void)
{
  /* Allocate memory for environment variables */
  env	= (char **)ws_calloc(MAX_CGI_ENVNO, sizeof(char **));
  envno = 0;

  /*
   * Set common environment variables
   */
  ws_asprintf(&env[envno++], "PATH=%s", DEFAULT_PATH);
  ws_asprintf(&env[envno++], "SERVER_SOFTWARE=%s", SERVER_VERSION);
  ws_asprintf(&env[envno++], "GATEWAY_INTERFACE=%s", CGI_VERSION);
  ws_asprintf(&env[envno++], "SERVER_ROOT=%s", httpd->serverRoot);
  DBG(fprintf(stderr, "[%s] init: %d\n", WM_MODULE_ID, envno);)
}

/*
 * Function Name:	wm_env_finish
 * Description:		clear server environment variables
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_env_finish(void)
{
  register int	i;

  for (i = 0; i < envno; i++) {
    wv_free(env[i]);
  }
  free(env);
}


/*
 * Function Name:	wm_set_script_environment
 * Description:		set script environments for CGI/SSI
 * Arguments:		message
 * Return Value:	Successful: 0, otherwise: -1
 */
static int
wm_set_script_environment(msg)
	register Message *	msg;
{
  register const char *	p;

#define	MAGIC_TYPE_ENV	"application/x-httpd-"
  if (ws_strncmp(msg->file.mime, MAGIC_TYPE_ENV, length(MAGIC_TYPE_ENV)) != 0 &&
      ws_strcmp(msg->file.mime, MAGIC_TYPE_SSI) != 0) {
    return WM_RET_OK;
  }
  DBG(fputs("<wm_set_script_environment>\n", stderr);)

  /* create CGI environments */
  msg->cgi.env = (char **)ws_calloc(MAX_CGI_ENVNO, sizeof(char **));
  for (msg->cgi.envno = 0; msg->cgi.envno < envno; msg->cgi.envno++) {
    msg->cgi.env[msg->cgi.envno] = ws_strdup(env[msg->cgi.envno]);
  }
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_PROTOCOL=%s",
	      msg->request.http.protocol);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_PORT=%d",
	      msg->soci.port);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_NAME=%s",
	      msg->vhost->serverName);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_ADMIN=%s",
	      msg->vhost->serverAdmin);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "DOCUMENT_ROOT=%s",
	      msg->vhost->documentRoot);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REQUEST_METHOD=%s",
	      msg->request.method.name);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SCRIPT_NAME=%s",
	      msg->request.uri);
  if (msg->request.path_info != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_INFO=%s",
		msg->request.path_info);

    if (msg->request.path_info[1] == '~') {
      size_t		len;
      char *		name;
      struct passwd *	pwd;

      len  = strcspn(msg->request.path_info    + TILDE_OFFSET, "/");
      name = ws_strndup(msg->request.path_info + TILDE_OFFSET, len);
      pwd  = getpwnam(name);
      if (pwd == NULL) {	/* invalid user: no home directory  */
	ws_asprintf(&msg->cgi.env[msg->cgi.envno++],
		    "PATH_TRANSLATED=%s/%%7E%s",
		    msg->vhost->documentRoot,
		    msg->request.path_info + TILDE_OFFSET);
      }
      else {
	ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_TRANSLATED=%s/%s%s",
		    pwd->pw_dir, msg->vhost->userDir,
		    msg->request.path_info + TILDE_OFFSET + len);
      }
      free((char *)name);
    }
    else {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_TRANSLATED=%s%s",
		  msg->vhost->documentRoot, msg->request.path_info);
    }
  }

  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "QUERY_STRING=%s",
	      msg->request.query_string == NULL ? ""
						: msg->request.query_string);

  if (msg->client.name != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_HOST=%s",
		msg->client.name);
  }

  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_ADDR=%s",
	      msg->client.ip);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_PORT=%s",
	      msg->client.port);

  if (msg->request.auth_type != NULL) {
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "AUTH_TYPE=%s",
	      msg->request.auth_type);
  }

  if (msg->user.authName != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_USER=%s",
		msg->user.authName);
  }

  if (msg->user.identName != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_IDENT=%s",
		msg->user.identName);
  }

  if (msg->request.method.type == M_POST) {
    p = wa_get(msg->request.header, "Content-Type");
    if (p != NULL) {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "CONTENT_TYPE=%s", p);
    }
    if (msg->request.content_length > 0) {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "CONTENT_LENGTH=%d",
		  msg->request.content_length);
    }
  }

  /* no standard environment variable */
  p = wa_get_all(msg->request.header, "Accept", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT=%s", p);
    free(p);
  }

  p = wa_get_all(msg->request.header, "Accept-Encoding", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT_ENCODING=%s", p);
    free(p);
  }

  p = wa_get_all(msg->request.header, "Accept-Language", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT_LANGUAGE=%s", p);
    free(p);
  }

  p = wa_get(msg->request.header, "Host");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_HOST=%s", p);
  }

  p = wa_get(msg->request.header, "User-Agent");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_USER_AGENT=%s", p);
  }

  p = wa_get(msg->request.header, "Referer");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_REFERER=%s", p);
  }

  p = wa_get(msg->request.header, "Cookie");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_COOKIE=%s", p);
  }

  p = wa_get(msg->request.header, "Cookie2");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_COOKIE2=%s", p);
  }

#if	ENABLE_SSL
  if (msg->soci.secure == SOCKET_SECURE) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_VERSION=%s",
		SSL_get_version(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER=%s",
		SSL_get_cipher(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER_NAME=%s",
		SSL_get_cipher_name(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER_VERSION=%s",
		SSL_get_cipher_version(wt_netfd_ssl(msg->nsd)));
  }
#endif	/* ENABLE_SSL */

  msg->cgi.env[msg->cgi.envno] = NULL;

#if	NODEBUG
  {
    register int	i;
    for (i = 0; i < msg->cgi.envno; i++) {
      fprintf(stderr, "[%02d] %s\n", i, msg->cgi.env[i]);
    }
  }
#endif	/* DEBUG */

  return WM_RET_OK;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_env_register_hooks(void)
{
  wm_hook_fixups(wm_set_script_environment, HOOK_FIXUP_REQUEST);
}

static wmModule	mod_env = {
  WM_MAGIC_NUMBER_MAJOR,	/* version */
  WM_UNSECURE,			/* unsecure */
  WM_MODULE_ID,			/* name */
  "Environment Variables Module version 1.1.6",	/* info */
  "Kouichi ABE (WALL)",		/* author */
  NULL,				/* depend */
  NULL,				/* handle */
  (void *)wm_env_init,		/* init */
  (void *)wm_env_finish,	/* finish */
  (void *)wm_env_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_env;
}

==> wyvern-2.2.10.tar.bz2: wyvern-2.2.10/modules/mod_env.c <==
/*****************************************************************************
 *
 * FILE:	mod_env.c
 * DESCRIPTION:	Wyvern: making environment variables module
 * DATE:	Tue, Aug 20 2002
 * UPDATED:	Wed, Sep 12 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * REFERENCE:	draft-coar-cgi-v11-03.txt
 * LICENSE:
 *
 *  Copyright (c) 2002-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_env.c,v 1.26 2007/09/12 02:24:26 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <pwd.h>
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structure definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_env"

#define	length(x)	(sizeof((x)) - 1)

/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	wm_env_init(void);
static void	wm_env_finish(void);
static int	wm_set_script_environment(Message *);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static char **	env;
static int	envno;

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_env_init
 * Description:		initialize server environment for CGI/SSI
 * Arguments:		none
 * Return Value:	none
 * Notes:
 *
 * SERVER_SOFTWARE:	the name and version of the server software
 * SERVER_NAME:		the name of the server
 * GATEWAY_INTERFACE:	the revision of the CGI specification
 *
 * SERVER_PROTOCOL:	the protocol version of the HTTP
 * SERVER_PORT:		the port which the request is received on the server
 *
 * REQUEST_METHOD:	the method of the HTTP
 * SCRIPT_NAME:		the identity of the CGI script
 * PATH_INFO:		the path to be interpreted by the CGI script
 * PATH_TRANSLATED:	the location in the server's document
 * QUERY_STRING:	a URL-encoded string
 * REMOTE_HOST:		the FQDN of the client
 * REMOTE_ADDR:		the IP address of the client
 * REMOTE_PORT:		the remote port of the client
 * AUTH_TYPE:		the requests "Authorization"
 * REMOTE_USER:		the remote user-ID
 * REMOTE_IDENT:	the identity information reported about the connection
 *			by a RFC 1413
 * CONTENT_TYPE:	the Internet Media Type
 * CONTENT_LENGTH:	the size of the message-body entity
 *
 *
 * HTTP_ACCEPT:
 * HTTP_ACCEPT_ENCODING:
 * HTTP_ACCEPT_LANGUAGE:
 * HTTP_HOST:
 * HTTP_USER_AGENT:
 * HTTP_REFERER:
 * HTTP_COOKIE:
 * HTTP_COOKIE2:
 *
 * DOCUMENT_ROOT:	NCSA and APACHE added -- not in CGI spec
 * SERVER_ROOT:		NCSA added
 * SERVER_ADMIN:	APACHE added
 */
static void
wm_env_init(void)
{
  /* Allocate memory for environment variables */
  env	= (char **)ws_calloc(MAX_CGI_ENVNO, sizeof(char **));
  envno = 0;

  /*
   * Set common environment variables
   */
  ws_asprintf(&env[envno++], "PATH=%s", DEFAULT_PATH);
  ws_asprintf(&env[envno++], "SERVER_SOFTWARE=%s", SERVER_VERSION);
  ws_asprintf(&env[envno++], "GATEWAY_INTERFACE=%s", CGI_VERSION);
  ws_asprintf(&env[envno++], "SERVER_ROOT=%s", httpd->serverRoot);
  DBG(fprintf(stderr, "[%s] init: %d\n", WM_MODULE_ID, envno);)
}

/*
 * Function Name:	wm_env_finish
 * Description:		clear server environment variables
 * Arguments:		none
 * Return Value:	none
 */
static void
wm_env_finish(void)
{
  register int	i;

  for (i = 0; i < envno; i++) {
    wv_free(env[i]);
  }
  free(env);
}


/*
 * Function Name:	wm_set_script_environment
 * Description:		set script environments for CGI/SSI
 * Arguments:		message
 * Return Value:	Successful: 0, otherwise: -1
 */
static int
wm_set_script_environment(msg)
	register Message *	msg;
{
  register const char *	p;

#define	MAGIC_TYPE_ENV	"application/x-httpd-"
  if (ws_strncmp(msg->file.mime, MAGIC_TYPE_ENV, length(MAGIC_TYPE_ENV)) != 0 &&
      ws_strcmp(msg->file.mime, MAGIC_TYPE_SSI) != 0) {
    return WM_RET_OK;
  }
  DBG(fputs("<wm_set_script_environment>\n", stderr);)

  /* create CGI environments */
  msg->cgi.env = (char **)ws_calloc(MAX_CGI_ENVNO, sizeof(char **));
  for (msg->cgi.envno = 0; msg->cgi.envno < envno; msg->cgi.envno++) {
    msg->cgi.env[msg->cgi.envno] = ws_strdup(env[msg->cgi.envno]);
  }
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_PROTOCOL=%s",
	      msg->request.http.protocol);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_PORT=%d",
	      msg->soci.port);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_NAME=%s",
	      msg->vhost->serverName);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SERVER_ADMIN=%s",
	      msg->vhost->serverAdmin);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "DOCUMENT_ROOT=%s",
	      msg->vhost->documentRoot);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REQUEST_METHOD=%s",
	      msg->request.method.name);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SCRIPT_NAME=%s",
	      msg->request.uri);
  if (msg->request.path_info != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_INFO=%s",
		msg->request.path_info);

    if (msg->request.path_info[1] == '~') {
      size_t		len;
      char *		name;
      struct passwd *	pwd;

      len  = strcspn(msg->request.path_info    + TILDE_OFFSET, "/");
      name = ws_strndup(msg->request.path_info + TILDE_OFFSET, len);
      pwd  = getpwnam(name);
      if (pwd == NULL) {	/* invalid user: no home directory  */
	ws_asprintf(&msg->cgi.env[msg->cgi.envno++],
		    "PATH_TRANSLATED=%s/%%7E%s",
		    msg->vhost->documentRoot,
		    msg->request.path_info + TILDE_OFFSET);
      }
      else {
	ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_TRANSLATED=%s/%s%s",
		    pwd->pw_dir, msg->vhost->userDir,
		    msg->request.path_info + TILDE_OFFSET + len);
      }
      free((char *)name);
    }
    else {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "PATH_TRANSLATED=%s%s",
		  msg->vhost->documentRoot, msg->request.path_info);
    }
  }

  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "QUERY_STRING=%s",
	      msg->request.query_string == NULL ? ""
						: msg->request.query_string);

  if (msg->client.name != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_HOST=%s",
		msg->client.name);
  }

  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_ADDR=%s",
	      msg->client.ip);
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_PORT=%s",
	      msg->client.port);

  if (msg->request.auth_type != NULL) {
  ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "AUTH_TYPE=%s",
	      msg->request.auth_type);
  }

  if (msg->user.authName != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_USER=%s",
		msg->user.authName);
  }

  if (msg->user.identName != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "REMOTE_IDENT=%s",
		msg->user.identName);
  }

  if (msg->request.method.type == M_POST) {
    p = wa_get(msg->request.header, "content-type");
    if (p != NULL) {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "CONTENT_TYPE=%s", p);
    }
    if (msg->request.content_length > 0) {
      ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "CONTENT_LENGTH=%d",
		  msg->request.content_length);
    }
  }

  /* no standard environment variable */
  p = wa_get_all(msg->request.header, "accept", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT=%s", p);
    free(p);
  }

  p = wa_get_all(msg->request.header, "accept-encoding", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT_ENCODING=%s", p);
    free(p);
  }

  p = wa_get_all(msg->request.header, "accept-language", ",");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_ACCEPT_LANGUAGE=%s", p);
    free(p);
  }

  p = wa_get(msg->request.header, "host");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_HOST=%s", p);
  }

  p = wa_get(msg->request.header, "user-agent");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_USER_AGENT=%s", p);
  }

  p = wa_get(msg->request.header, "referer");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_REFERER=%s", p);
  }

  p = wa_get(msg->request.header, "cookie");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_COOKIE=%s", p);
  }

  p = wa_get(msg->request.header, "cookie2");
  if (p != NULL) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "HTTP_COOKIE2=%s", p);
  }

#if	ENABLE_SSL
  if (msg->soci.secure == SOCKET_SECURE) {
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_VERSION=%s",
		SSL_get_version(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER=%s",
		SSL_get_cipher(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER_NAME=%s",
		SSL_get_cipher_name(wt_netfd_ssl(msg->nsd)));
    ws_asprintf(&msg->cgi.env[msg->cgi.envno++], "SSL_CIPHER_VERSION=%s",
		SSL_get_cipher_version(wt_netfd_ssl(msg->nsd)));
  }
#endif	/* ENABLE_SSL */

  msg->cgi.env[msg->cgi.envno] = NULL;

#if	NODEBUG
  {
    register int	i;
    for (i = 0; i < msg->cgi.envno; i++) {
      fprintf(stderr, "[%02d] %s\n", i, msg->cgi.env[i]);
    }
  }
#endif	/* DEBUG */

  return WM_RET_OK;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_env_register_hooks(void)
{
  wm_hook_fixups(wm_set_script_environment, HOOK_FIXUP_REQUEST);
}

static wmModule	mod_env = {
  WM_MAGIC_NUMBER_MAJOR,	/* version */
  WM_UNSECURE,			/* unsecure */
  WM_MODULE_ID,			/* name */
  "Environment Variables Module version 1.1.6",	/* info */
  "Kouichi ABE (WALL)",		/* author */
  NULL,				/* depend */
  NULL,				/* handle */
  (void *)wm_env_init,		/* init */
  (void *)wm_env_finish,	/* finish */
  (void *)wm_env_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_env;
}
